/**
 * 
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.activemq.bug;

import EDU.oswego.cs.dl.util.concurrent.CountDown;

import org.activemq.ActiveMQConnectionFactory;

import javax.jms.Connection;
import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;

import junit.framework.TestCase;

/**
 * Test case for AMQ-336
 * 
 * @author Matthew Vincent 
 * @version $Revision: 1.1 $
 */
public class AMQ336Test extends TestCase {
    CountDown outstandingMessages = new CountDown(3);
    Inbox in;

    class Inbox implements MessageListener {
        private final ActiveMQConnectionFactory connectionFactory;

        private Connection connection;
        private Session session;
        private Queue destination;
        private MessageProducer producer;
        private MessageConsumer consumer;
        private final boolean durable = false;
        private String clientID = "clientid";

        public Inbox() throws JMSException {
            this.connectionFactory = new ActiveMQConnectionFactory("vm://localhost");
            connection = connectionFactory.createConnection();
            if (durable && clientID != null) {
                connection.setClientID(clientID);
            }
            connection.start();
            session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
            destination = session.createQueue("subject");
            producer = session.createProducer(destination);
            if (durable) {
                producer.setDeliveryMode(DeliveryMode.PERSISTENT);
            }
            else {
                producer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
            }
            consumer = session.createConsumer(destination);
        }

        public void stop() throws JMSException {
            if (consumer != null)
                consumer.close();
            if (session != null)
                session.close();
            if (connection != null)
                connection.close();
        }

        public void receive(int message) throws JMSException {
            Message jmsMsg = session.createObjectMessage(new Integer(message));
            // jmsMsg.setJMSPriority(message);
            producer.send(jmsMsg);
        }

        public void startDeliveringMessages() throws JMSException {
            consumer.setMessageListener(this);
        }

        public void onMessage(Message arg0) {
            outstandingMessages.release();
        }

    }

    protected void setUp() throws Exception {
        in = new Inbox();
    }

    protected void tearDown() throws Exception {
        if (in != null)
            in.stop();
    }

    public void testDeadlock() throws Exception {
        in.receive(2);
        in.receive(1);
        in.receive(3);
        in.startDeliveringMessages();
        outstandingMessages.attempt(5000);
        assertEquals(0, outstandingMessages.currentCount());
    }

}
