/*
 * Copyright (c) 2005 Your Corporation. All Rights Reserved.
 */
package org.activemq.transport.stomp;

import junit.framework.TestCase;
import org.activemq.broker.BrokerContainer;
import org.activemq.broker.impl.BrokerContainerImpl;
import org.activemq.message.ConnectionInfo;
import org.activemq.message.Receipt;
import org.activemq.message.SessionInfo;

import javax.jms.JMSException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;

public class StompWireFormatTest extends TestCase
{
    private StompWireFormat wire;

    public void setUp() throws Exception
    {
        wire = new StompWireFormat();
    }

    public void testValidConnectHandshake() throws Exception
    {
        String connect_frame = "CONNECT\n" +
                               "login: brianm\n" +
                               "passcode: wombats\n" +
                               "\n" +
                               Stomp.NULL;
        DataInputStream din = new DataInputStream(new ByteArrayInputStream(connect_frame.getBytes()));
        ByteArrayOutputStream bout = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream(bout);

        wire.registerTransportStreams(dout, din);
        wire.initiateServerSideProtocol();


        ConnectionInfo ci = (ConnectionInfo) wire.readPacket(din);
        assertNotNull(ci);
        assertTrue(ci.isReceiptRequired());

        Receipt cr = new Receipt();
        cr.setCorrelationId(ci.getId());
        wire.writePacket(cr, dout);

        SessionInfo si = (SessionInfo) wire.readPacket(null);
        assertNotNull(si);
        assertTrue(si.isReceiptRequired());

        Receipt sr = new Receipt();
        sr.setCorrelationId(si.getId());
        wire.writePacket(sr, dout);

        String response = new String(bout.toByteArray());
        assertTrue(response.startsWith("CONNECTED"));
    }

    public void _testFakeServer() throws Exception
    {
        final BrokerContainer container = new BrokerContainerImpl();
        new Thread(new Runnable()
        {
            public void run()
            {
                try
                {
                    container.addConnector("tcp://127.0.0.1:61613", new StompWireFormat());
                    container.start();
                }
                catch (JMSException e)
                {
                    System.err.println("ARGH");
                }
            }
        }).start();
        System.err.println("started container");
        System.err.println("okay, go play");

        System.err.println(System.in.read());
    }
}
