/*
 * Created on 12-Aug-2005
 */
package org.activemq.transport.https;

import java.net.URI;

import javax.jms.JMSException;

import org.activemq.transport.http.HttpTransportConnector;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.mortbay.http.SslListener;

public class HttpsTransportConnector extends HttpTransportConnector {

	private static final Log log = LogFactory.getLog( HttpsTransportConnector.class );

	private String keyPassword = System.getProperty( "javax.net.ssl.keyPassword" );
	private String keyStorePassword = System.getProperty( "javax.net.ssl.keyStorePassword" );
	private String keyStore = System.getProperty( "javax.net.ssl.keyStore" );
	private String keyStoreType = null;
	private String certificateAlgorithm = null;
	private String protocol = null;
	
	public HttpsTransportConnector( URI uri ) {
		super( uri );
	}

	public void start() throws JMSException {
		SslListener sslListener = new SslListener();
		sslListener.setKeystore( keyStore );
		sslListener.setPassword( keyStorePassword );
		// if the keyPassword hasn't been set, default it to the
		// key store password
		if ( keyPassword == null ) {
			sslListener.setKeyPassword( keyStorePassword );	
		}
		if ( keyStoreType != null ) {
			sslListener.setKeystoreType( keyStoreType );
		}
		if ( certificateAlgorithm != null ) {
			sslListener.setAlgorithm( certificateAlgorithm );
		}
		if ( protocol != null ) {
			sslListener.setProtocol( protocol );
		}
		
		setSocketListener( sslListener );
		
		super.start();
	}
	
	// Properties
	//--------------------------------------------------------------------------------
	
	public String getCertificateAlgorithm() {
		return certificateAlgorithm;
	}

	public void setCertificateAlgorithm( String certificateAlgorithm ) {
		this.certificateAlgorithm = certificateAlgorithm;
	}

	public String getKeyStore() {
		return keyStore;
	}

	public void setKeyStore( String keyStore ) {
		this.keyStore = keyStore;
	}

	public String getKeyPassword() {
		return keyPassword;
	}

	public void setKeyPassword( String keyPassword ) {
		this.keyPassword = keyPassword;
	}

	public String getKeyStoreType() {
		return keyStoreType;
	}

	public void setKeyStoreType( String keyStoreType ) {
		this.keyStoreType = keyStoreType;
	}

	public String getKeyStorePassword() {
		return keyStorePassword;
	}

	public void setKeyStorePassword( String keyStorePassword ) {
		this.keyStorePassword = keyStorePassword;
	}

	public String getProtocol() {
		return protocol;
	}

	public void setProtocol( String protocol ) {
		this.protocol = protocol;
	}

}
