/** 
 * 
 * Copyright 2005 Guillaume Nodet
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.activemq.ra;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.resource.ResourceException;
import javax.resource.spi.endpoint.MessageEndpoint;

import org.activemq.ActiveMQConnection;
import org.activemq.ActiveMQConnectionFactory;
import org.activemq.ActiveMQSession;
import org.activemq.io.util.MemoryBoundedQueue;
import org.activemq.message.ActiveMQTextMessage;
import org.activemq.transport.TransportChannel;
import org.jmock.MockObjectTestCase;
import org.jmock.cglib.Mock;

/**
 * @version $Revision: 1.1.1.1 $
 */
public class ServerSessionImplTest extends MockObjectTestCase {

	public void testBatch() throws Exception {
		DummyActiveMQConnection connection = new DummyActiveMQConnection(new ActiveMQConnectionFactory(), 
				 null, 
				 null, 
				 getMockTransportChannel());
		ServerSessionPoolImpl pool = new ServerSessionPoolImpl(null, 1);
		DummyActiveMQSession session = new DummyActiveMQSession(connection);
		MemoryBoundedQueue queue = connection.getMemoryBoundedQueue("Session(" + session.getSessionId() + ")");
		queue.enqueue(new ActiveMQTextMessage());
		queue.enqueue(new ActiveMQTextMessage());
		queue.enqueue(new ActiveMQTextMessage());
		DummyMessageEndpoint endpoint = new DummyMessageEndpoint();
		ServerSessionImpl serverSession = new ServerSessionImpl(pool, session, null, endpoint, true, 2);
		serverSession.run();
		assertEquals(2, endpoint.messagesPerBatch.size());
		assertEquals(new Integer(2), endpoint.messagesPerBatch.get(0));
		assertEquals(new Integer(1), endpoint.messagesPerBatch.get(1));
	}
	
	private class DummyMessageEndpoint implements MessageEndpoint, MessageListener {
		protected List messagesPerBatch = new ArrayList();
		protected int nbMessages = -1000;
		public void beforeDelivery(Method arg0) throws NoSuchMethodException, ResourceException {
			nbMessages = 0;
		}
		public void afterDelivery() throws ResourceException {
			messagesPerBatch.add(new Integer(nbMessages));
			nbMessages = -1000;
		}
		public void release() {
		}
		public void onMessage(Message arg0) {
			nbMessages ++;
		}
	}
	
	private class DummyActiveMQSession extends ActiveMQSession {
		public DummyActiveMQSession(ActiveMQConnection connection) throws JMSException {
			super(connection, 0);
		}
		public short getSessionId() {
			return super.getSessionId();
		}
	}
	
	private class DummyActiveMQConnection extends ActiveMQConnection {
		public DummyActiveMQConnection(ActiveMQConnectionFactory factory,
				String theUserName, String thePassword,
				TransportChannel transportChannel) throws JMSException {
			super(factory, theUserName, thePassword, transportChannel);
		}
		public MemoryBoundedQueue getMemoryBoundedQueue(String name) {
			return super.getMemoryBoundedQueue(name);
		}
	}
	
	private TransportChannel getMockTransportChannel() {
		Mock tc = new Mock(TransportChannel.class);
		tc.expects(once()).method("setPacketListener");
		tc.expects(once()).method("setExceptionListener");
		tc.expects(once()).method("addTransportStatusEventListener");
		tc.expects(atLeastOnce()).method("asyncSend");
		tc.expects(atLeastOnce()).method("send");
		return (TransportChannel) tc.proxy();
	}
}
